from typing import Any, List

import httpx

from healthpy._http import _check, _is_json


class _Request:
    def __init__(self, url: str, **args):
        with httpx.Client() as client:
            self.response = client.get(url, timeout=args.pop("timeout", (1, 5)), **args)

    def is_error(self) -> bool:
        return self.response.is_error

    def content(self) -> Any:
        return (
            self.response.json()
            if _is_json(self.response.headers.get("content-type"))
            else self.response.text
        )


def check(
    service_name: str,
    url: str,
    status_extracting: callable = None,
    failure_status: str = None,
    affected_endpoints: List[str] = None,
    additional_keys: dict = None,
    error_status_extracting: callable = None,
    **httpx_args,
) -> (str, dict):
    """
    Return Health "Checks object" for an external service connection.

    :param service_name: External service name.
    :param url: External service health check URL.
    :param status_extracting: Function returning status according to the JSON or text response (as parameter).
    Default to the way status should be extracted from a service following healthcheck RFC.
    :param error_status_extracting: Function returning status according to the JSON or text response (as parameter).
    Default to the way status should be extracted from a service following healthcheck RFC or fail_status.
    Note that the response might be None as this is called to extract the default status in case of failure as well.
    :param affected_endpoints: List of endpoints affected if dependency is down. Default to None.
    :param additional_keys: Additional user defined keys to send in checks.
    :return: A tuple with a string providing the status (amongst healthpy.*_status variable) and the "Checks object".
    Based on https://inadarei.github.io/rfc-healthcheck/
    """
    return _check(
        service_name=service_name,
        url=url,
        request_class=_Request,
        status_extracting=status_extracting,
        failure_status=failure_status,
        affected_endpoints=affected_endpoints,
        additional_keys=additional_keys,
        error_status_extracting=error_status_extracting,
        **httpx_args,
    )



import datetime
import re
from typing import List, Any, Optional
import warnings

import healthpy


def _is_json(content_type: Optional[str]) -> bool:
    return re.match(r"application/(health\+)?json", content_type or "") is not None


def _api_health_status(health_response: Any) -> str:
    if isinstance(health_response, dict):
        return health_response.get("status", healthpy.pass_status)
    return healthpy.pass_status


def _api_error_health_status(health_response: Any) -> str:
    if isinstance(health_response, dict):
        return health_response.get("status", healthpy.fail_status)
    return healthpy.fail_status


def safe_error_status_extracting(error_status_extracting: callable) -> str:
    try:
        return error_status_extracting(None)
    except:
        return healthpy.fail_status


def _check(
    service_name: str,
    url: str,
    request_class,
    status_extracting: callable = None,
    failure_status: str = None,
    affected_endpoints: List[str] = None,
    additional_keys: dict = None,
    error_status_extracting: callable = None,
    **kwargs,
) -> (str, dict):
    """
    Return Health "Checks object" for an external service connection.

    :param service_name: External service name.
    :param url: External service health check URL.
    :param status_extracting: Function returning status according to the JSON or text response (as parameter).
    Default to the way status should be extracted from a service following healthcheck RFC or pass_status.
    :param error_status_extracting: Function returning status according to the JSON or text response (as parameter).
    Default to the way status should be extracted from a service following healthcheck RFC or fail_status.
    Note that the response might be None as this is called to extract the default status in case of failure as well.
    :param affected_endpoints: List of endpoints affected if dependency is down. Default to None.
    :param additional_keys: Additional user defined keys to send in checks.
    :return: A tuple with a string providing the status (amongst healthpy.*_status variable) and the "Checks object".
    Based on https://inadarei.github.io/rfc-healthcheck/
    """
    try:
        request = request_class(url, **kwargs)
        response = request.content()
        if request.is_error():
            if not error_status_extracting:
                error_status_extracting = _api_error_health_status

            if failure_status:
                warnings.warn(
                    "failure_status is deprecated and should not be used anymore. Use error_status_extracting instead.",
                    DeprecationWarning,
                )

            status = failure_status or error_status_extracting(response)
            check = {"output": response} if status != healthpy.pass_status else {}
        else:
            if not status_extracting:
                status_extracting = _api_health_status

            status = status_extracting(response)
            check = {"observedValue": response}
    except Exception as e:
        if failure_status:
            warnings.warn(
                "failure_status is deprecated and should not be used anymore. Use error_status_extracting instead.",
                DeprecationWarning,
            )
        status = failure_status or safe_error_status_extracting(error_status_extracting)
        check = {"output": str(e)} if status != healthpy.pass_status else {}

    if affected_endpoints and status != healthpy.pass_status:
        check["affectedEndpoints"] = affected_endpoints

    if additional_keys:
        check.update(additional_keys)

    return (
        status,
        {
            f"{service_name}:health": {
                "componentType": url,
                "status": status,
                "time": datetime.datetime.utcnow().isoformat(),
                **check,
            }
        },
    )



import json
from typing import Callable, Union
import asyncio

import flask_restx
import flask

import healthpy
from healthpy._response import response_body, consul_response_status_code


def add_consul_health_endpoint(
    namespace: Union[flask_restx.Namespace, flask_restx.Api],
    health_check: Callable,
    **kwargs
):
    """
    Create /health: Consul Health check endpoint implementing https://inadarei.github.io/rfc-healthcheck/ but following
    Consul expected status code (https://www.consul.io/docs/agent/checks.html).

    :param namespace: The Flask-RestX namespace.
    :param health_check: async callable returning a tuple of size 2 with a string providing the status (pass, warn, fail)
    and the "Checks object" as a dictionary as per https://inadarei.github.io/rfc-healthcheck/
    :param version: (optional) public version of the service. If not provided, version will be extracted from the
    release_id, considering that release_id is following semantic versioning.
    Version will be considered as the MAJOR component of a MAJOR.MINOR.PATCH release_id.
    :param release_id: (optional) in well-designed APIs, backwards-compatible changes in the service
    should not update a version number. APIs usually change their version number as infrequently as possible,
    to preserve stable interface. However, implementation of an API may change much more frequently,
    which leads to the importance of having separate “release number” or “releaseId”
    that is different from the public version of the API.
    :param notes: (optional) array of notes relevant to current state of health.
    :param links: (optional) is an object containing link relations and URIs [RFC3986]
    for external links that MAY contain more information about the health of the endpoint.
    All values of this object SHALL be URIs. Keys MAY also be URIs.
    Per web-linking standards [RFC8288] a link relationship SHOULD either be a common/registered one
    or be indicated as a URI, to avoid name clashes.
    If a “self” link is provided, it MAY be used by clients to check health via HTTP response code, as mentioned above.
    :param service_id: (optional) is a unique identifier of the service, in the application scope.
    :param description: (optional) is a human-friendly description of the service.
    """

    @namespace.route("/health")
    @namespace.doc(
        responses={
            200: (
                "Server is in a coherent state.",
                namespace.model(
                    "HealthPass",
                    {
                        "status": flask_restx.fields.String(
                            description="Indicates whether the service status is acceptable or not.",
                            required=True,
                            example="pass",
                            enum=["pass"],
                        ),
                        "version": flask_restx.fields.String(
                            description="Public version of the service.",
                            required=True,
                            example="1",
                        ),
                        "releaseId": flask_restx.fields.String(
                            description="Version of the service.",
                            required=True,
                            example="1.0.0",
                        ),
                        "checks": flask_restx.fields.Raw(
                            description="Provides more details about the status of the service.",
                            required=True,
                        ),
                    },
                ),
            ),
            429: (
                "Server is almost in a coherent state.",
                namespace.model(
                    "HealthWarn",
                    {
                        "status": flask_restx.fields.String(
                            description="Indicates whether the service status is acceptable or not.",
                            required=True,
                            example="warn",
                            enum=["warn"],
                        ),
                        "version": flask_restx.fields.String(
                            description="Public version of the service.",
                            required=True,
                            example="1",
                        ),
                        "releaseId": flask_restx.fields.String(
                            description="Version of the service.",
                            required=True,
                            example="1.0.0",
                        ),
                        "checks": flask_restx.fields.Raw(
                            description="Provides more details about the status of the service.",
                            required=True,
                        ),
                    },
                ),
            ),
            400: (
                "Server is not in a coherent state.",
                namespace.model(
                    "HealthFail",
                    {
                        "status": flask_restx.fields.String(
                            description="Indicates whether the service status is acceptable or not.",
                            required=True,
                            example="fail",
                            enum=["fail"],
                        ),
                        "version": flask_restx.fields.String(
                            description="Public version of the service.",
                            required=True,
                            example="1",
                        ),
                        "releaseId": flask_restx.fields.String(
                            description="Version of the service.",
                            required=True,
                            example="1.0.0",
                        ),
                        "checks": flask_restx.fields.Raw(
                            description="Provides more details about the status of the service.",
                            required=True,
                        ),
                        "output": flask_restx.fields.String(
                            description="Raw error output.", required=False
                        ),
                    },
                ),
            ),
        }
    )
    class Health(flask_restx.Resource):
        def get(self):
            """
            Check service health.
            This endpoint perform a quick server state check.
            """
            try:
                status, checks = asyncio.run(health_check())
                body = response_body(status, checks=checks, **kwargs)
            except Exception as e:
                status = healthpy.fail_status
                body = response_body(status, output=str(e), **kwargs)
            return flask.Response(
                json.dumps(body),
                status=consul_response_status_code(status),
                content_type="application/health+json",
            )



import warnings
from healthpy.requests import check

warnings.warn(
    "Importing healthpy.http is deprecated and will be removed in the future. Import healthpy.requests instead.",
    DeprecationWarning,
)



from datetime import datetime

import redis

import healthpy


def check(url: str, key_pattern: str, additional_keys: dict = None) -> (str, dict):
    """
    Return Health "Checks object" for redis keys.

    :param url: Redis URL
    :param key_pattern: Pattern to look for in keys.
    :param additional_keys: Additional user defined keys to send in checks.
    :return: A tuple with a string providing the status (amongst healthpy.*_status variable) and the "Checks object".
    Based on https://inadarei.github.io/rfc-healthcheck/
    """
    additional_keys = additional_keys or {}
    try:
        redis_server = redis.Redis.from_url(url)
        redis_server.ping()

        keys = redis_server.keys(key_pattern)

        if not keys or not isinstance(keys, list):
            return (
                healthpy.fail_status,
                {
                    "redis:ping": {
                        "componentType": "component",
                        "status": healthpy.fail_status,
                        "time": datetime.utcnow().isoformat(),
                        "output": f"{key_pattern} cannot be found in {keys}",
                        **additional_keys,
                    }
                },
            )

        return (
            healthpy.pass_status,
            {
                "redis:ping": {
                    "componentType": "component",
                    "observedValue": f"{key_pattern} can be found.",
                    "status": healthpy.pass_status,
                    "time": datetime.utcnow().isoformat(),
                    **additional_keys,
                }
            },
        )
    except Exception as e:
        return (
            healthpy.fail_status,
            {
                "redis:ping": {
                    "componentType": "component",
                    "status": healthpy.fail_status,
                    "time": datetime.utcnow().isoformat(),
                    "output": str(e),
                    **additional_keys,
                }
            },
        )
